pub enum SigningKeyStatus {
SignAndVerify,
VerifyOnly,
}Expand description
Indicates the status of a SigningKey.
Variants
SignAndVerify
The key may be used to both sign new tokens, and verify existing tokens.
VerifyOnly
The key may be used only to verify existing tokens. New tokens will never be signed using a key with this status. This allows you to deprecate a key you intend to replace.